import sqlite3
import json
from pathlib import Path
import os
import sys
import traceback

def get_latest_calibration_file(directory):
    """Find the most recent LED calibration file in the given directory."""
    try:
        calibration_files = [f for f in os.listdir(directory) if f.startswith('led_calibration_') and f.endswith('.sqlite')]
        if not calibration_files:
            raise FileNotFoundError(f"No calibration files found in {directory}")
        
        latest_file = max(calibration_files, key=lambda f: os.path.getmtime(os.path.join(directory, f)))
        return os.path.join(directory, latest_file)
    except Exception as e:
        print(f"Error in get_latest_calibration_file: {e}")
        raise

def main():
    try:
        # Set the directory where calibration files are stored
        calibration_dir = Path("/home/pioreactor/calibrations/spectrometer")

        # Get the latest calibration file or use the provided one
        if len(sys.argv) > 1:
            db_path = sys.argv[1]
        else:
            db_path = get_latest_calibration_file(calibration_dir)

        print(f"Using calibration file: {db_path}")

        # Read the SQLite database
        conn = sqlite3.connect(db_path)
        cursor = conn.cursor()

        # Fetch all data
        cursor.execute("SELECT LED, LED_Name, Intensity, Repeat, Wavelength, Reading FROM calibration_data")
        data = cursor.fetchall()

        # Close the connection
        conn.close()

        # Process the data
        led_data = {}
        for led, led_name, intensity, repeat, wavelength, reading in data:
            if led not in led_data:
                led_data[led] = {'name': led_name, 'intensities': {}}
            if intensity not in led_data[led]['intensities']:
                led_data[led]['intensities'][intensity] = []
            led_data[led]['intensities'][intensity].append((wavelength, reading))

        # Calculate the max non-saturated intensity for each LED
        max_reading = max(reading for _, _, _, _, _, reading in data)
        saturation_threshold = 0.99 * max_reading

        led_max_intensities = {}
        for led in led_data:
            for intensity in sorted(led_data[led]['intensities'].keys(), reverse=True):
                max_intensity_reading = max(reading for _, reading in led_data[led]['intensities'][intensity])
                if max_intensity_reading < saturation_threshold:
                    led_max_intensities[led] = intensity
                    break

        # Prepare data for the chart
        chart_data = []
        for led in led_data:
            max_intensity = led_max_intensities[led]
            wavelengths, readings = zip(*sorted(led_data[led]['intensities'][max_intensity]))
            chart_data.append({
                'name': led_data[led]['name'],
                'intensity': max_intensity,
                'wavelengths': wavelengths,
                'readings': readings
            })

        # Create HTML content
        html_content = f"""
        <!DOCTYPE html>
        <html lang="en">
        <head>
            <meta charset="UTF-8">
            <meta name="viewport" content="width=device-width, initial-scale=1.0">
            <title>LED Spectrum Chart</title>
            <script src="https://cdn.jsdelivr.net/npm/chart.js"></script>
        </head>
        <body>
            <canvas id="ledChart" width="800" height="400"></canvas>
            <script>
                const data = {json.dumps(chart_data)};
                const ctx = document.getElementById('ledChart').getContext('2d');
                new Chart(ctx, {{
                    type: 'line',
                    data: {{
                        datasets: data.map(led => ({{
                            label: `${{led.name}} (Intensity: ${{led.intensity}}%)`,
                            data: led.wavelengths.map((w, i) => ({{ x: w, y: led.readings[i] }})),
                            borderColor: `hsl(${{Math.random() * 360}}, 100%, 50%)`,
                            fill: false
                        }}))
                    }},
                    options: {{
                        responsive: true,
                        title: {{
                            display: true,
                            text: 'LED Spectrum Chart (Max Non-Saturating Intensity for Each LED)'
                        }},
                        scales: {{
                            x: {{
                                type: 'linear',
                                position: 'bottom',
                                title: {{
                                    display: true,
                                    text: 'Wavelength (nm)'
                                }}
                            }},
                            y: {{
                                title: {{
                                    display: true,
                                    text: 'Intensity (AU)'
                                }}
                            }}
                        }}
                    }}
                }});
            </script>
        </body>
        </html>
        """

        # Save the HTML file
        output_filename = f"led_spectrum_chart_{Path(db_path).stem.split('_', 2)[-1]}.html"
        output_path = calibration_dir / output_filename
        with open(output_path, 'w') as f:
            f.write(html_content)

        print(f"Chart saved to: {output_path}")

    except Exception as e:
        print(f"Error in main function: {e}")
        print("Traceback:")
        traceback.print_exc()
        sys.exit(1)

if __name__ == "__main__":
    main()