# -*- coding: utf-8 -*-
from __future__ import annotations

import adafruit_as7341
import board
import click
import sqlite3
from typing import List, Dict
import os
from pathlib import Path
from datetime import datetime
import time
import subprocess
import pioreactor.actions.led_intensity as led_utils
from pioreactor import types as pt
from pioreactor.background_jobs.base import BackgroundJobWithDodgingContrib
from pioreactor.background_jobs.leader.mqtt_to_db_streaming import produce_metadata
from pioreactor.background_jobs.leader.mqtt_to_db_streaming import register_source_to_sink
from pioreactor.background_jobs.leader.mqtt_to_db_streaming import TopicToParserToTable
from pioreactor.config import config
from pioreactor.exc import HardwareNotFoundError
from pioreactor.utils.timing import current_utc_datetime
from pioreactor.whoami import get_latest_experiment_name
from pioreactor.whoami import get_unit_name

# Define the wavelengths and their corresponding LED channels
WAVELENGTH_LED_MAP = {
    415: config.get("spectrometer_reading.config", "led_415", fallback="builtin"),
    445: config.get("spectrometer_reading.config", "led_445", fallback="builtin"),
    480: config.get("spectrometer_reading.config", "led_480", fallback="builtin"),
    515: config.get("spectrometer_reading.config", "led_515", fallback="builtin"),
    555: config.get("spectrometer_reading.config", "led_555", fallback="builtin"),
    590: config.get("spectrometer_reading.config", "led_590", fallback="builtin"),
    630: config.get("spectrometer_reading.config", "led_630", fallback="builtin"),
    680: config.get("spectrometer_reading.config", "led_680", fallback="builtin"),
}
# Get LED names from config
LED_NAMES = {key: value for key, value in config['leds'].items()}

# Map wavelengths to AS7341 channel names
WAVELENGTH_TO_CHANNEL = {
    415: 'f1',
    445: 'f2',
    480: 'f3',
    515: 'f4',
    555: 'f5',
    590: 'f6',
    630: 'f7',
    680: 'f8'
}

def parser(topic: str, payload: pt.MQTTMessagePayload) -> dict:
    metadata = produce_metadata(topic)

    return {
        "experiment": metadata.experiment,
        "pioreactor_unit": metadata.pioreactor_unit,
        "timestamp": current_utc_datetime(),
        "reading": float(payload),
        "band": int(metadata.rest_of_topic[-1].removeprefix("band_")),
    }

register_source_to_sink(
    [
        TopicToParserToTable(
            f"pioreactor/+/+/spectrometer_reading/band_{band}",
            parser,
            "as7341_spectrum_readings",
        ) for band in WAVELENGTH_LED_MAP.keys()
    ]
)

class SpectrometerReading(BackgroundJobWithDodgingContrib):
    job_name = "spectrometer_reading"
    
    def __init__(self, unit: str, experiment: str) -> None:
        super().__init__(unit=unit, experiment=experiment, plugin_name="spectrometer_reading_plugin")

        try:
            i2c = board.I2C()
            self.sensor = adafruit_as7341.AS7341(i2c)
            self.logger.info("AS7341 sensor initialized successfully")
            
            self.configure_smux()
            
            self.sensor.led_current = config.getfloat("spectrometer_reading.config", "led_current_mA", fallback=5)
            self.sensor.gain = 10
            time.sleep(0.1)  # Allow time for gain to be set
            
            self.logger.debug(f"Sensor ID: {self.sensor._device_id}")
            self.logger.debug(f"Sensor gain: {self.sensor.gain}")
            self.logger.debug(f"Sensor LED current: {self.sensor.led_current}")
            self.logger.debug(f"Sensor atime: {self.sensor.atime}")
            self.logger.debug(f"Sensor astep: {self.sensor.astep}")
            
        except Exception as e:
            self.logger.error(f"Error initializing AS7341: {str(e)}")
            raise HardwareNotFoundError("Is the AS7341 board attached to the Pioreactor HAT?")

        self.wavelength_led_map = {
            415: config.get("spectrometer_reading.config", "led_415", fallback="builtin"),
            445: config.get("spectrometer_reading.config", "led_445", fallback="builtin"),
            480: config.get("spectrometer_reading.config", "led_480", fallback="builtin"),
            515: config.get("spectrometer_reading.config", "led_515", fallback="builtin"),
            555: config.get("spectrometer_reading.config", "led_555", fallback="builtin"),
            590: config.get("spectrometer_reading.config", "led_590", fallback="builtin"),
            630: config.get("spectrometer_reading.config", "led_630", fallback="builtin"),
            680: config.get("spectrometer_reading.config", "led_680", fallback="builtin"),
        }
        
        self.wavelength_led_intensity_map = {
            415: config.getfloat("spectrometer_reading.config", "led_415_intensity", fallback=100.0),
            445: config.getfloat("spectrometer_reading.config", "led_445_intensity", fallback=100.0),
            480: config.getfloat("spectrometer_reading.config", "led_480_intensity", fallback=100.0),
            515: config.getfloat("spectrometer_reading.config", "led_515_intensity", fallback=100.0),
            555: config.getfloat("spectrometer_reading.config", "led_555_intensity", fallback=100.0),
            590: config.getfloat("spectrometer_reading.config", "led_590_intensity", fallback=100.0),
            630: config.getfloat("spectrometer_reading.config", "led_630_intensity", fallback=100.0),
            680: config.getfloat("spectrometer_reading.config", "led_680_intensity", fallback=100.0),
        }



        self.check_ir_led_and_reference()
        self.check_led_functionality()
        self.logger.info(f"Sensor LED current set to {self.sensor.led_current}mA, gain set to {self.sensor.gain}")
        self._background_noise = [0.0] * 8
        self.record_background_noise()


    @property
    def led_on_and_rest_off_state(self) -> dict[pt.LedChannel, pt.LedIntensityValue]:
        # Turn on all LEDs used for spectrometer readings
        return {
            channel: (self.led_intensity if channel in set(self.wavelength_led_map.values()) else 0.0)
            for channel in led_utils.ALL_LED_CHANNELS
        }

    def configure_smux(self):
        self.logger.debug("Configuring SMUX")
        self.sensor._configure_f1_f4()
        time.sleep(0.1)
        self.sensor._configure_f5_f8()
        time.sleep(0.1)
        self.logger.debug("SMUX configuration complete")

    def check_led_functionality(self):
        self.logger.info("Checking LED functionality")
        for wavelength, led_channel in self.wavelength_led_map.items():
            self.logger.debug(f"Testing LED for wavelength {wavelength}")
            self.activate_led_for_wavelength(wavelength)
            time.sleep(1)  # Keep LED on for 1 second for visual confirmation
            self.deactivate_leds()
            time.sleep(0.5)  # Short pause between LED tests

    def check_ir_led_and_reference(self):
        self.logger.debug("Checking IR LED and reference photodiode")
        
        # Deactivate all LEDs
        self.deactivate_leds()
        
        # Read background
        background = self.read_all_channels()
        self.logger.debug(f"Background readings: {background}")
        
        # Turn on IR LED
        ir_led_channel = self.wavelength_led_map[680]  # Assuming 680nm is the IR LED
        ir_led_intensity = self.wavelength_led_intensity_map[680]
        
        self.activate_led_for_wavelength(680)
        
        ir_readings = self.read_all_channels()
        self.logger.debug(f"IR LED on readings: {ir_readings}")
        
        # Deactivate LEDs again
        self.deactivate_leds()
        
        # Calculate the difference
        diff_readings = [ir - bg for ir, bg in zip(ir_readings, background)]
        self.logger.debug(f"Difference readings: {diff_readings}")
        
        # Check if the reference photodiode (channel 1) is responding
        ref_channel_index = 0  # Assuming channel 1 is the first in the list
        if diff_readings[ref_channel_index] <= 0:
            self.logger.error("Reference photodiode not responding to IR LED. Check connections and positioning.")
        else:
            self.logger.info(f"Reference photodiode response: {diff_readings[ref_channel_index]}")
        
        # Check other channels for unexpected responses
        for i, diff in enumerate(diff_readings):
            if i != ref_channel_index and diff > diff_readings[ref_channel_index] / 2:
                self.logger.warning(f"Channel {i+1} has unexpectedly high response to IR LED: {diff}")
        
        return diff_readings

    def action_to_do_before_od_reading(self) -> None:
        self.logger.debug("Pausing spectrometer readings for OD measurement")
        # Add any necessary actions to pause spectrometer readings

    def action_to_do_after_od_reading(self) -> None:
        self.logger.debug("Resuming spectrometer readings after OD measurement")
        # Add any necessary actions to resume spectrometer readings
    def log_sensor_status(self):
        self.logger.debug(f"Sensor ID: {self.sensor._device_id}")
        self.logger.debug(f"Sensor gain: {self.sensor.gain}")
        self.logger.debug(f"Sensor LED current: {self.sensor.led_current}")
        self.logger.debug(f"Sensor atime: {self.sensor.atime}")
        self.logger.debug(f"Sensor astep: {self.sensor.astep}")
        self.logger.debug(f"SMUX enabled: {self.sensor._smux_enabled}")
        self.logger.debug(f"Power enabled: {self.sensor._power_enabled}")
        self.logger.debug(f"LED control enabled: {self.sensor._led_control_enabled}")       

    def get_led_state_for_wavelength(self, wavelength: int) -> dict[pt.LedChannel, pt.LedIntensityValue]:
        led_channel = self.wavelength_led_map.get(wavelength)
        led_intensity = self.wavelength_led_intensity_map.get(wavelength, 100.0)
        
        if led_channel is None:
            self.logger.warning(f"No LED mapped for wavelength {wavelength}")
            return {channel: 0.0 for channel in led_utils.ALL_LED_CHANNELS}
        
        led_state = {channel: 0.0 for channel in led_utils.ALL_LED_CHANNELS}
        
        if led_channel == "builtin":
            self.sensor.led = True
            self.sensor.led_current = int(led_intensity / 100 * config.getfloat("spectrometer_reading.config", "led_current_mA", fallback=5))
            self.logger.debug(f"Activated builtin LED for wavelength {wavelength} with current {self.sensor.led_current}mA")
        else:
            led_state[led_channel] = led_intensity
            self.logger.debug(f"Setting external LED {led_channel} to {led_intensity}% for wavelength {wavelength}")
        
        return led_state

    def activate_led_for_wavelength(self, wavelength: int):
        led_state = self.get_led_state_for_wavelength(wavelength)
        self.logger.debug(f"Activating LED state for wavelength {wavelength}: {led_state}")
        
        led_channel = self.wavelength_led_map.get(wavelength)
        if led_channel != "builtin":
            led_utils.led_intensity(
                led_state,
                unit=self.unit,
                experiment=self.experiment,
                source_of_event=self.job_name,
                pubsub_client=self.pub_client,
            )
        
        # Small delay to ensure LED is fully on
        time.sleep(0.1)

    def deactivate_leds(self):
        self.logger.debug("Deactivating all LEDs")
        try:
            led_utils.led_intensity(
                {channel: 0.0 for channel in led_utils.ALL_LED_CHANNELS},
                unit=self.unit,
                experiment=self.experiment,
                source_of_event=self.job_name,
                pubsub_client=self.pub_client,
            )
            self.sensor.led = False
            self.logger.debug("All LEDs deactivated")
        except Exception as e:
            self.logger.error(f"Error in deactivate_leds: {str(e)}")
        time.sleep(0.1)  # Small delay to ensure LEDs are fully off


    def record_background_noise(self) -> None:
        self.logger.debug("Starting record_background_noise method")
        with led_utils.change_leds_intensities_temporarily(
            {channel: 0.0 for channel in led_utils.ALL_LED_CHANNELS},
            unit=self.unit,
            experiment=self.experiment,
            source_of_event=self.job_name,
            pubsub_client=self.pub_client,
            verbose=False,
        ):
            time.sleep(0.1)
            
            raw_background = self.read_all_channels()
            self.logger.debug(f"Raw background readings: {raw_background}")
            
            if all(reading == 0 for reading in raw_background):
                self.logger.warning("All background readings are zero. This might indicate a sensor issue.")
            
            self._background_noise = self.normalize_by_gain_time(raw_background)
            self.logger.debug(f"Normalized background noise: {self._background_noise}")

    def normalize_by_gain_time(self, band_recordings: list[int]) -> list[float]:
        self.logger.debug(f"normalize_by_gain_time called with: {band_recordings}")
        integration_time = (self.sensor.atime + 1) * (self.sensor.astep + 1) * 2.78e-6  # in seconds
        gain_value = self.sensor.gain.value if hasattr(self.sensor.gain, 'value') else self.sensor.gain
        self.logger.debug(f"Integration time: {integration_time} seconds")
        self.logger.debug(f"Gain: {gain_value}")
        
        normalized = [
            (x / gain_value / integration_time if x is not None else None)
            for x in band_recordings
        ]
        self.logger.debug(f"After normalization: {normalized}")
        return normalized

    def normalize_by_offset(self, band_recording: float, wavelength: int) -> float:
        if band_recording is None:
            return None
        
        self.logger.debug(f"normalize_by_offset called with band_recording={band_recording}, wavelength={wavelength}")
        
        wavelength_to_index = {
            415: 0, 445: 1, 480: 2, 515: 3,
            555: 4, 590: 5, 630: 6, 680: 7
        }
        
        if wavelength in wavelength_to_index:
            index = wavelength_to_index[wavelength]
            self.logger.debug(f"Normalizing for wavelength {wavelength}, index {index}")
            self.logger.debug(f"Background noise array: {self._background_noise}")
            
            if 0 <= index < len(self._background_noise):
                normalized = band_recording - self._background_noise[index]
                self.logger.debug(f"Normalized value: {normalized}")
                return normalized
            else:
                self.logger.warning(f"Index {index} out of range for _background_noise array. Using raw value.")
        else:
            self.logger.warning(f"Unexpected wavelength {wavelength}. Using raw value.")
        
        return band_recording
    
    def continuous_reading(self, interval=5):
        self.logger.info(f"Starting continuous reading with {interval} second interval")
        while self.state == self.READY:
            try:
                readings = self.record_all_bands()
                for wavelength, reading in zip(self.wavelength_led_map.keys(), readings):
                    if reading is not None:
                        self.publish(
                            f"pioreactor/{self.unit}/{self.experiment}/spectrometer_reading/band_{wavelength}",
                            reading
                        )
                    else:
                        self.logger.warning(f"No valid reading for wavelength {wavelength}")
            except Exception as e:
                self.logger.error(f"Error in continuous reading: {str(e)}")
                self.logger.debug(f"Error details:", exc_info=True)
            time.sleep(interval)

    def record_all_bands(self) -> list[float]:
        self.logger.debug("Starting record_all_bands method")
        readings = []
        wavelengths = list(self.wavelength_led_map.keys())
        
        for wavelength in wavelengths:
            channel_name = WAVELENGTH_TO_CHANNEL[wavelength]
            led_channel = self.wavelength_led_map[wavelength]
            led_intensity = self.wavelength_led_intensity_map[wavelength]
            self.logger.debug(f"Reading channel: {channel_name}, using LED: {led_channel} at intensity: {led_intensity}%")
            
            self.activate_led_for_wavelength(wavelength)
            
            try:
                raw_value = getattr(self.sensor, channel_name)
                self.logger.debug(f"Raw reading for {channel_name}: {raw_value}")
                readings.append(raw_value)
            except AttributeError as e:
                self.logger.error(f"Error accessing {channel_name}: {str(e)}")
                readings.append(None)
            
            self.deactivate_leds()

        if all(reading is None for reading in readings):
            self.logger.warning("All readings are None. This might indicate a sensor issue.")

        gain_normalized_readings = self.normalize_by_gain_time(readings)
        self.logger.debug(f"Gain normalized readings: {gain_normalized_readings}")

        offset_normalized_readings = [
            self.normalize_by_offset(reading, wavelength)
            for reading, wavelength in zip(gain_normalized_readings, wavelengths)
        ]
        self.logger.debug(f"Offset normalized readings: {offset_normalized_readings}")

        return offset_normalized_readings
    def record_specific_bands(self, bands: list[int]) -> dict[int, float]:
        self.logger.debug(f"Starting record_specific_bands method for bands: {bands}")
        readings = {}
        valid_wavelengths = list(self.wavelength_led_map.keys())
        
        for wavelength in bands:
            if wavelength not in valid_wavelengths:
                self.logger.warning(f"Unsupported wavelength: {wavelength}")
                continue

            channel_name = WAVELENGTH_TO_CHANNEL[wavelength]
            led_channel = self.wavelength_led_map[wavelength]
            led_intensity = self.wavelength_led_intensity_map[wavelength]
            self.logger.debug(f"Reading channel: {channel_name}, using LED: {led_channel} at intensity: {led_intensity}%")
            
            self.activate_led_for_wavelength(wavelength)
            
            try:
                raw_value = getattr(self.sensor, channel_name)
                self.logger.debug(f"Raw reading for {channel_name}: {raw_value}")
                
                gain_normalized = self.normalize_by_gain_time([raw_value])[0]
                offset_normalized = self.normalize_by_offset(gain_normalized, wavelength)
                
                readings[wavelength] = offset_normalized
                setattr(self, f"band_{wavelength}", offset_normalized)
                self.logger.debug(f"Normalized reading for {wavelength}nm: {offset_normalized}")
            except AttributeError as e:
                self.logger.error(f"Error accessing {channel_name}: {str(e)}")
                readings[wavelength] = None
                self.logger.warning(f"No valid reading for {wavelength}nm")
            
            self.deactivate_leds()

        if all(reading == 0 for reading in readings.values() if reading is not None):
            self.logger.warning("All readings are zero. This might indicate a sensor issue.")

        self.logger.debug(f"Final readings: {readings}")
        return readings

    def read_all_channels(self):
        channels = ['415nm', '445nm', '480nm', '515nm', '555nm', '590nm', '630nm', '680nm']
        readings = []
        for channel in channels:
            value = getattr(self.sensor, f'channel_{channel}')
            readings.append(value)
            self.logger.debug(f"Channel {channel}: {value}")
        return readings

    def diagnostic_read(self):
        self.logger.debug("Performing diagnostic read...")
        try:
            self.sensor.initialize()
            self.logger.debug("Sensor re-initialized")
            self.configure_smux()
            
            # Verify I2C communication
            device_id = self.sensor._device_id
            self.logger.debug(f"Device ID: {device_id}")
            
            # Test with built-in LED on
            self.sensor.led = True
            self.sensor.led_current = 100  # Set to maximum
            time.sleep(0.1)
            
            for gain in [adafruit_as7341.Gain.GAIN_0_5, adafruit_as7341.Gain.GAIN_1, 
                        adafruit_as7341.Gain.GAIN_2, adafruit_as7341.Gain.GAIN_4, 
                        adafruit_as7341.Gain.GAIN_8, adafruit_as7341.Gain.GAIN_16, 
                        adafruit_as7341.Gain.GAIN_32, adafruit_as7341.Gain.GAIN_64, 
                        adafruit_as7341.Gain.GAIN_128, adafruit_as7341.Gain.GAIN_256, 
                        adafruit_as7341.Gain.GAIN_512]:
                self.sensor.gain = gain
                self.sensor.integration_time = 200  # Increase integration time
                time.sleep(0.1)
                self.logger.debug(f"Testing with gain: {gain}")
                readings = self.read_all_channels()
                self.logger.debug(f"Readings with gain {gain}: {readings}")
            
            # Turn off LED
            self.sensor.led = False
            
            # Reset to original settings
            self.sensor.gain = adafruit_as7341.Gain.GAIN_10
            self.sensor.integration_time = 100
            time.sleep(0.1)
            
            self.log_sensor_status()
            
        except Exception as e:
            self.logger.error(f"Error during diagnostic read: {str(e)}")

    def calibrate_leds(self, intensities: List[float] = None, repeats: int = 3) -> Dict:
        if intensities is None:
            intensities = [0, 10, 20, 30, 40, 50, 60, 70, 80, 90, 100]
        
        self.logger.debug(f"Starting LED calibration with intensities: {intensities} and {repeats} repeats")
        
        calibration_data = {}
        all_leds = set(WAVELENGTH_LED_MAP.values())
        all_wavelengths = list(WAVELENGTH_LED_MAP.keys())

        self.logger.debug(f"All LEDs to calibrate: {all_leds}")
        self.logger.debug(f"All wavelengths to measure: {all_wavelengths}")

        for led in all_leds:
            led_name = LED_NAMES.get(led, led)
            self.logger.debug(f"Calibrating LED: {led_name}")
            calibration_data[led] = {}
            for intensity in intensities:
                self.logger.debug(f"  Testing intensity: {intensity}%")
                calibration_data[led][intensity] = []
                for repeat in range(repeats):
                    self.logger.debug(f"    Repeat: {repeat + 1}")
                    if led != "builtin":
                        led_utils.led_intensity(
                            {led: intensity},
                            unit=self.unit,
                            experiment=self.experiment,
                            source_of_event=self.job_name,
                            pubsub_client=self.pub_client,
                        )
                    else:
                        self.sensor.led = True
                        self.sensor.led_current = intensity

                    time.sleep(0.1)  # Allow LED to stabilize
                    readings = {}
                    for wavelength in all_wavelengths:
                        channel_name = WAVELENGTH_TO_CHANNEL[wavelength]
                        raw_channel = getattr(self.sensor, channel_name)
                        normalized_channel = self.normalize_by_gain_time([raw_channel])[0]
                        normalized_channel = self.normalize_by_offset(normalized_channel, wavelength)
                        readings[wavelength] = normalized_channel
                    
                    self.logger.debug(f"      Readings: {readings}")
                    calibration_data[led][intensity].append(readings)

                    if led != "builtin":
                        led_utils.led_intensity(
                            {led: 0},
                            unit=self.unit,
                            experiment=self.experiment,
                            source_of_event=self.job_name,
                            pubsub_client=self.pub_client,
                        )
                    else:
                        self.sensor.led = False

        self.logger.debug("LED calibration complete")
        return calibration_data

    def save_calibration_data(self, calibration_data: Dict) -> str:
        calibration_dir = Path("/home/pioreactor/calibrations/spectrometer")
        calibration_dir.mkdir(parents=True, exist_ok=True)
        
        timestamp = datetime.now().strftime('%Y%m%d_%H%M%S')
        db_filename = f"led_calibration_{timestamp}.sqlite"
        full_path = calibration_dir / db_filename

        conn = sqlite3.connect(str(full_path))
        cursor = conn.cursor()

        cursor.execute('''
            CREATE TABLE calibration_data
            (LED TEXT, LED_Name TEXT, Intensity REAL, Repeat INTEGER, Wavelength INTEGER, Reading REAL)
        ''')

        for led, intensities in calibration_data.items():
            led_name = LED_NAMES.get(led, led)
            for intensity, repeats in intensities.items():
                for repeat_index, readings in enumerate(repeats):
                    for wavelength, reading in readings.items():
                        cursor.execute('''
                            INSERT INTO calibration_data
                            (LED, LED_Name, Intensity, Repeat, Wavelength, Reading)
                            VALUES (?, ?, ?, ?, ?, ?)
                        ''', (led, led_name, intensity, repeat_index + 1, wavelength, reading))

        conn.commit()
        conn.close()

        self.logger.info(f"Calibration data saved to {full_path}")
        return str(full_path)

    def generate_spectrum_chart(self, db_file_path: str):
        try:
            script_path = Path(__file__).parent / "led-spectrum-chart.py"
            result = subprocess.run(["python", str(script_path), db_file_path], 
                                    check=True, capture_output=True, text=True)
            self.logger.info("LED spectrum chart generated successfully.")
            self.logger.info(result.stdout)  # Log the output of the chart generation script
        except subprocess.CalledProcessError as e:
            self.logger.error(f"Failed to generate LED spectrum chart. Exit code: {e.returncode}")
            self.logger.error(f"Error output:\n{e.output}")
            self.logger.error(f"Standard output:\n{e.stdout}")
        except Exception as e:
            self.logger.error(f"Unexpected error in generate_spectrum_chart: {e}")
            self.logger.error("Traceback:")
            self.logger.error(traceback.format_exc())

@click.command(name="spectrometer_reading")
@click.option("--bands", "-b", multiple=True, type=int, help="Specific wavelengths to measure. Can be used multiple times.")
@click.option("--all", "measure_all", is_flag=True, help="Measure all available bands")
@click.option("--continuous", "-c", is_flag=True, help="Run continuously")
@click.option("--calibrate", is_flag=True, help="Run LED calibration")
@click.option("--interval", "-i", type=float, default=5.0, help="Interval between readings in continuous mode (seconds)")

def click_spectrometer_reading(bands, measure_all, continuous, calibrate, interval):
    """
    Start spectrometer reading from the AS7341 sensor.
    """
    job = SpectrometerReading(
        unit=get_unit_name(),
        experiment=get_latest_experiment_name(),
    )

    if calibrate:
        # Add calibration logic here if needed
        job.logger.info("Calibration not implemented yet.")
        job.clean_up()
        return

    if not bands and not measure_all and not continuous:
        # Default behavior: run continuously for all bands
        job.continuous_reading(interval)
    elif measure_all:
        bands = [415, 445, 480, 515, 555, 590, 630, 680]

    if continuous:
        job.continuous_reading(interval)
    elif bands:
        readings = job.record_specific_bands(bands)
        for wavelength, value in readings.items():
            click.echo(f"Band {wavelength}: {value}")
        job.clean_up()

    # Add diagnostic read
    #job.diagnostic_read()

    job.block_until_disconnected()

if __name__ == "__main__":
    click_spectrometer_reading()